///////////////////////////////////////////////
//                                           //
// VRLab Mental Vision Module 11: Particles  //
// Author: Loc Duboux - Achille Peternier   //
// Copyright: EPFL-VRLab 2007 (C)            //
//                                           //
///////////////////////////////////////////////

#include "../data/common.h"
#include <tinyxml.h>
#include <mvisio.h>
#include <SDL/SDL.h>
#include <SDL/SDL_syswm.h>
#include <string>
#include <vector>

#include <stdio.h>
#include <iostream>
#include <fstream>
#include <iomanip>

using namespace std;

/***************************/
/* Preprocessing variables */
/***************************/
#define MODULE_NAME "Particle engine"
#define ENERGY_SLIDER 22
#define SIZE_SLIDER ENERGY_SLIDER+16
#define WEIGHT_SLIDER SIZE_SLIDER+16
#define SPIN_SLIDER WEIGHT_SLIDER+16
#define COLOR_RANGE_SLIDER 118

#define UP_VECTOR_SLIDER 37

#define NB_PART_SLIDER	37
#define SPINNING_SLIDER NB_PART_SLIDER+32
#define POSITION_SLIDER SPINNING_SLIDER+32


/********************/
/* External library */
/********************/
#ifdef _DEBUG
#pragma comment(lib, "mvisio_d.lib")
#pragma comment(lib, "tinyxml_d.lib")
#else
#pragma comment(lib, "mvisio.lib")
#pragma comment(lib, "tinyxml.lib")
#endif
#pragma comment(lib, "sdlmain.lib")
#pragma comment(lib, "sdl.lib")


/********************/
/* Global variables */
/********************/
OPENFILENAME	ofn;
char*			fileName = NULL;

float minPosX, maxPosX;
float minPosY, maxPosY;
float minPosZ, maxPosZ;
float maxEnergy;
float minRadius, maxRadius;
float minPlanarX, maxPlanarX;
float minPlanarY, maxPlanarY;
float minParticleSize, maxParticleSize;

// Creation of a Vector of emitter
MVEMITTER*			emitter;
bool				isEmitterInitialized = false;
vector<MVEMITTER*> vecEmitters;
unsigned idSelectedEmitter = 0;

/****************************/
/* Blending Windows options */
/****************************/

// Set blending Window
MVWINDOW *blendingWin;
MVBUTTON *blendingSrcClicked;
MVBUTTON *blendingDstClicked;
MVBUTTON *foobar;
MVBUTTON *gl_zeroSrcButton;
MVBUTTON *gl_zeroDstButton;
MVBUTTON *gl_oneSrcButton;
MVBUTTON *gl_oneDstButton;
MVBUTTON *gl_colorSrcButton;
MVBUTTON *gl_colorDstButton;
MVBUTTON *gl_src_SrcAlphaButton;
MVBUTTON *gl_dst_SrcAlphaButton;
MVBUTTON *gl_src_DstAlphaButton;
MVBUTTON *gl_dst_DstAlphaButton;
MVBUTTON *gl_oneMinusDstColorButton;
MVBUTTON *gl_oneMinusSrcColorButton;
MVBUTTON *gl_src_oneMinusSrcAlphaButton;
MVBUTTON *gl_dst_oneMinusSrcAlphaButton;
MVBUTTON *gl_src_oneMinusDstAlphaButton;
MVBUTTON *gl_dst_oneMinusDstAlphaButton;
MVBUTTON *gl_SrcAlphaSaturateButton;

/************************/
/* Functions prototypes */
/************************/
void setBlendingOptions(int src, int dst);
bool loadSetup(const char* filename);
bool clean();							// Cleaning all objects
bool saveEffectAsXML(char *filename);	// Save all the emitters into an XML file
bool saveEmitterAsXML(char *filename);	// Save the selected emitter into an XML file

/*****************/
/* Main function */
/*****************/
int main(int argc, char *argv[]) //
{
	minPosX		= -100.0f;
	minPosY		= -100.0f;
	minPosZ		= -100.0f;
	maxPosX		= 100.0f;
	maxPosY		= 100.0f;
	maxPosZ		= 100.0f;
	maxEnergy   = 200.0f;
	minPlanarX	= 1.0f;
	minPlanarY	= 1.0f;
	maxPlanarX	= 100.0f;
	maxPlanarY	= 100.0f;
	minRadius	= 1.0f;
	maxRadius	= 100.0f;
	minParticleSize = 0.0f;
	maxParticleSize = 10.0f;

		if(!loadSetup("data/ParticleEditorSetup.txt"))	// Load the setup of the editor
		return -1;								// This is required for choosing the min/max value of the position sliders

	// for opening/saving file
	SDL_SysWMinfo info;
	SDL_VERSION(&info.version);
	SDL_GetWMInfo(&info);
	HWND hWnd = info.window;

	fileName = (char *) malloc(sizeof(char) * 256);
	sprintf(fileName, "\0");

	memset(&ofn, 0, sizeof (ofn));
	ofn.lStructSize  = sizeof (OPENFILENAME);
	ofn.lpstrInitialDir = TEXT("C:\\");
	ofn.hwndOwner = hWnd;
	ofn.nMaxFile = MAX_PATH;
	ofn.lpstrFile = fileName;
	
	/////////////////
   // Init MVisio //
   /////////////////
   INIT_MVISIO(MODULE_NAME, argc, argv);   
   
	
	/*********************************/
	/* Creation of a particle System */
	/*********************************/
	
	// Creation d'une fentre
	USE_FONT();
	USE_BACKGROUND(MODULE_SIZEX, MODULE_SIZEY);
   USE_QUITBUTTON(MODULE_SIZEX, MODULE_SIZEY);    	
	USE_CAMERA(0.0f, 0.0f, 200.0f, 0.0f, 0.0f, 0.0f); 
	
	// Chargement d'une texture pour le moteur de particule
	MVTEXTURE *tex = new MVTEXTURE();
	tex->load("particle.tga");
	
	// Button to display the panels
	MVBUTTON *displayToolboxPanelButton = new MVBUTTON();
	displayToolboxPanelButton->setFont(font);
	displayToolboxPanelButton->setCaption("^x0035^y0005Toolbox");
	displayToolboxPanelButton->setSizes(120, 30);
	displayToolboxPanelButton->setPosition(30, 30);

	MVBUTTON *displaySysParametersButton = new MVBUTTON();
	displaySysParametersButton->setFont(font);
	displaySysParametersButton->setCaption("^y0005 System parameters");
	displaySysParametersButton->setSizes(120, 30);
	displaySysParametersButton->setPosition(170, 30);

	MVBUTTON *displayParticuleParametersButton = new MVBUTTON();
	displayParticuleParametersButton->setFont(font);
	displayParticuleParametersButton->setCaption("^y0005Particle parameters");
	displayParticuleParametersButton->setSizes(120, 30);
	displayParticuleParametersButton->setPosition(310, 30);

	MVBUTTON *extraParametersButton = new MVBUTTON();
	extraParametersButton->setFont(font);
	extraParametersButton->setCaption("^x0008^y0005Extra parameters");
	extraParametersButton->setSizes(120, 30);
	extraParametersButton->setPosition(450, 30);

	// Toolbox Window
	bool isCreateEmitterWinDisplayed = false;
	MVWINDOW *toolboxWin = new MVWINDOW();
	toolboxWin->setFont(font);
	toolboxWin->setSizes(150, 210);
	toolboxWin->setCaption("Toolbox");
	toolboxWin->setTexture(baseTexture);
	toolboxWin->setPosition(30.0f, 80.0f); 
	
	
	// System Parameters Window
	bool isSystemParameterPanelDisplayed = false;
	MVWINDOW *parametersWin = new MVWINDOW();
	parametersWin->setFont(font);
	parametersWin->setSizes(245, 357); 
	parametersWin->setCaption("System parameters");
	parametersWin->setTexture(baseTexture);
	parametersWin->setPosition(170.0f, 80.0f);

	// Blending Window
	bool isBlendingPanelDisplayed = false;
	blendingWin = new MVWINDOW();
	blendingWin->setFont(font);
	blendingWin->setSizes(415, 290); 
	blendingWin->setCaption("Blending options");
	blendingWin->setTexture(baseTexture);
	blendingWin->setPosition(170.0f, 455.0f);

	// Particlue Parameters Window
	bool isParticuleParamsWinDisplayed = false;
	MVWINDOW *particuleParamsWin = new MVWINDOW();
	particuleParamsWin->setFont(font);
	particuleParamsWin->setSizes(250, 258); 
	particuleParamsWin->setCaption("Particle parameters");
	particuleParamsWin->setTexture(baseTexture);
	particuleParamsWin->setPosition(310.0f, 80.0f);

	// Extra Parameters Window (for planar and circular emitter)
	bool isExtraParamsWinDisplayed = false;
	MVWINDOW *extraParamsWin = new MVWINDOW();
	extraParamsWin->setFont(font);
	extraParamsWin->setSizes(250, 145); 
	extraParamsWin->setCaption("Extra parameters");
	extraParamsWin->setTexture(baseTexture);
	extraParamsWin->setPosition(450, 80.0f); 


	// Buttons for the Toolbox Panel
	MVBUTTON *createNewSphericalButton = new MVBUTTON();
	createNewSphericalButton->setFont(font);
	createNewSphericalButton->setCaption("^y0005New spherical emitter");
	createNewSphericalButton->setSizes(130, 30);
	createNewSphericalButton->setPosition(10, 10);
	createNewSphericalButton->link(toolboxWin);

	MVBUTTON *createNewPlanarButton = new MVBUTTON();
	createNewPlanarButton->setFont(font);
	createNewPlanarButton->setCaption("^y0005New planar emitter");
	createNewPlanarButton->setSizes(130, 30);
	createNewPlanarButton->setPosition(10, 50);
	createNewPlanarButton->link(toolboxWin);

	MVBUTTON *createNewCircularButton = new MVBUTTON();
	createNewCircularButton->setFont(font);
	createNewCircularButton->setCaption("^y0005New circular emitter");
	createNewCircularButton->setSizes(130, 30);
	createNewCircularButton->setPosition(10, 90);
	createNewCircularButton->link(toolboxWin);

	MVBUTTON *loadButton = new MVBUTTON();
	loadButton->setFont(font);
	loadButton->setCaption("^y0005Load emitter...");
	loadButton->setSizes(130, 30);
	loadButton->setPosition(10, 130);
	loadButton->link(toolboxWin);

	MVBUTTON *saveButton = new MVBUTTON();
	saveButton->setFont(font);
	saveButton->setCaption("^y0005Save emitter...");
	saveButton->setSizes(130, 30);
	saveButton->setPosition(10, 170);
	saveButton->setDisplayStatus(false);	
	saveButton->link(toolboxWin);
	
	

	// Buttons for the Particule Parameters Panel
	string Energy_text	= "\n^x0016Energy: %d\n";
	MVHSCROLLBAR *energyScroll = new MVHSCROLLBAR();
	energyScroll->link(particuleParamsWin);
	energyScroll->setSizeX(120);
	energyScroll->setPosition(103, ENERGY_SLIDER);
	energyScroll->setMinValue(0.0f);
	energyScroll->setMaxValue(maxEnergy);
	energyScroll->setValue(10.0f);

	string Size_text	= "^x0016Size: %d\n"; 
	MVHSCROLLBAR *partSizeScroll = new MVHSCROLLBAR();
	partSizeScroll->link(particuleParamsWin);
	partSizeScroll->setSizeX(120);
	partSizeScroll->setPosition(103, SIZE_SLIDER);
	partSizeScroll->setMinValue(minParticleSize);
	partSizeScroll->setMaxValue(maxParticleSize);
	partSizeScroll->setValue(100.0f);

	string Velocity_text	= "^x0016Velocity: %d\n";
	MVHSCROLLBAR *velocityScroll = new MVHSCROLLBAR();
	velocityScroll->link(particuleParamsWin);
	velocityScroll->setSizeX(120);
	velocityScroll->setPosition(103, WEIGHT_SLIDER);
	velocityScroll->setMinValue(1.0f);
	velocityScroll->setMaxValue(200.0f);
	velocityScroll->setValue(100.0f);

	string Spin_text	= "^x0016Spin: %d \n";
	MVHSCROLLBAR *partSpinScroll = new MVHSCROLLBAR();
	partSpinScroll->link(particuleParamsWin);
	partSpinScroll->setSizeX(120);
	partSpinScroll->setPosition(103, SPIN_SLIDER);
	partSpinScroll->setMinValue(1.0f);
	partSpinScroll->setMaxValue(200.0f);
	partSpinScroll->setValue(1.0f);

	string colorStart	= "\n^x0016Start color\n";
	string colorEnd	= "\n^x0016End color\n";
	string colorRange_r		= "^x0016R: %.2f\n";
	string colorRange_g		= "^x0016G: %.2f\n";
	string colorRange_b		= "^x0016B: %.2f\n";
	MVHSCROLLBAR *colorMinRedScroll = new MVHSCROLLBAR();
	colorMinRedScroll->link(particuleParamsWin);
	colorMinRedScroll->setSizeX(120);
	colorMinRedScroll->setPosition(103, COLOR_RANGE_SLIDER);
	colorMinRedScroll->setMinValue(0.0f);
	colorMinRedScroll->setMaxValue(1.0f);
	colorMinRedScroll->setValue(1.0f);

	MVHSCROLLBAR *colorMinGreenScroll = new MVHSCROLLBAR();
	colorMinGreenScroll->link(particuleParamsWin);
	colorMinGreenScroll->setSizeX(120);
	colorMinGreenScroll->setPosition(103, COLOR_RANGE_SLIDER+15);
	colorMinGreenScroll->setMinValue(0.0f);
	colorMinGreenScroll->setMaxValue(1.0f);
	colorMinGreenScroll->setValue(1.0f);

	MVHSCROLLBAR *colorMinBlueScroll = new MVHSCROLLBAR();
	colorMinBlueScroll->link(particuleParamsWin);
	colorMinBlueScroll->setSizeX(120);
	colorMinBlueScroll->setPosition(103, COLOR_RANGE_SLIDER+30);
	colorMinBlueScroll->setMinValue(0.0f);
	colorMinBlueScroll->setMaxValue(1.0f);
	colorMinBlueScroll->setValue(1.0f);

	MVHSCROLLBAR *colorMaxRedScroll = new MVHSCROLLBAR();
	colorMaxRedScroll->link(particuleParamsWin);
	colorMaxRedScroll->setSizeX(120);
	colorMaxRedScroll->setPosition(103, COLOR_RANGE_SLIDER+80);
	colorMaxRedScroll->setMinValue(0.0f);
	colorMaxRedScroll->setMaxValue(1.0f);
	colorMaxRedScroll->setValue(1.0f);

	MVHSCROLLBAR *colorMaxGreenScroll = new MVHSCROLLBAR();
	colorMaxGreenScroll->link(particuleParamsWin);
	colorMaxGreenScroll->setSizeX(120);
	colorMaxGreenScroll->setPosition(103, COLOR_RANGE_SLIDER+95);
	colorMaxGreenScroll->setMinValue(0.0f);
	colorMaxGreenScroll->setMaxValue(1.0f);
	colorMaxGreenScroll->setValue(1.0f);

	MVHSCROLLBAR *colorMaxBlueScroll = new MVHSCROLLBAR();
	colorMaxBlueScroll->link(particuleParamsWin);
	colorMaxBlueScroll->setSizeX(120);
	colorMaxBlueScroll->setPosition(103, COLOR_RANGE_SLIDER+110);
	colorMaxBlueScroll->setMinValue(0.0f);
	colorMaxBlueScroll->setMaxValue(1.0f);
	colorMaxBlueScroll->setValue(1.0f);

	// Write text on the colorRange Window
	particuleParamsWin->clear();
	particuleParamsWin->write((char*)Energy_text.c_str(), (int)energyScroll->getValue());
	particuleParamsWin->write((char*)Size_text.c_str(), (int)partSizeScroll->getValue());
	particuleParamsWin->write((char*)Velocity_text.c_str(), (int)velocityScroll->getValue());
	particuleParamsWin->write((char*)Spin_text.c_str(), (int)partSpinScroll->getValue());
	particuleParamsWin->write((char*)colorStart.c_str());
	particuleParamsWin->write((char*)colorRange_r.c_str(), colorMinRedScroll->getValue());
	particuleParamsWin->write((char*)colorRange_g.c_str(), colorMinGreenScroll->getValue());
	particuleParamsWin->write((char*)colorRange_b.c_str(), colorMinBlueScroll->getValue());
	particuleParamsWin->write((char*)colorEnd.c_str());
	particuleParamsWin->write((char*)colorRange_r.c_str(), colorMaxRedScroll->getValue());
	particuleParamsWin->write((char*)colorRange_g.c_str(), colorMaxGreenScroll->getValue());
	particuleParamsWin->write((char*)colorRange_b.c_str(), colorMaxBlueScroll->getValue());
	particuleParamsWin->format();


	// Extra Parameters Window
	string upVecStr		= "\n^x0016Up vector\n";
	string upVec_x			= "^x0016X: %.2f\n";
	string upVec_y			= "^x0016Y: %.2f\n";
	string upVec_z			= "^x0016Z: %.2f\n";
	MVHSCROLLBAR *upVectorScrollX = new MVHSCROLLBAR();
	upVectorScrollX->link(extraParamsWin);
	upVectorScrollX->setSizeX(130);
	upVectorScrollX->setPosition(93, UP_VECTOR_SLIDER);
	upVectorScrollX->setMinValue(-1.0f);
	upVectorScrollX->setMaxValue(1.0f);
	upVectorScrollX->setValue(0.0f);

	MVHSCROLLBAR *upVectorScrollY = new MVHSCROLLBAR();
	upVectorScrollY->link(extraParamsWin);
	upVectorScrollY->setSizeX(130);
	upVectorScrollY->setPosition(93, UP_VECTOR_SLIDER+15);
	upVectorScrollY->setMinValue(-1.0f);
	upVectorScrollY->setMaxValue(1.0f);
	upVectorScrollY->setValue(1.0f);

	MVHSCROLLBAR *upVectorScrollZ = new MVHSCROLLBAR();
	upVectorScrollZ->link(extraParamsWin);
	upVectorScrollZ->setSizeX(130);
	upVectorScrollZ->setPosition(93, UP_VECTOR_SLIDER+30);
	upVectorScrollZ->setMinValue(-1.0f);
	upVectorScrollZ->setMaxValue(1.0f);
	upVectorScrollZ->setValue(0.0f);

	string size_x			= "\n^x0016Size X: %.2f";
	string size_y			= "\n^x0016Size Y: %.2f\n";
	MVHSCROLLBAR *planarSizeXScroll = new MVHSCROLLBAR();
	planarSizeXScroll->setSizeX(100);
	planarSizeXScroll->setPosition(123, UP_VECTOR_SLIDER+64);
	planarSizeXScroll->setMinValue(minPlanarX);
	planarSizeXScroll->setMaxValue(maxPlanarY);
	planarSizeXScroll->setValue(10.0f);

	MVHSCROLLBAR *planarSizeYScroll = new MVHSCROLLBAR();
	planarSizeYScroll->setSizeX(100);
	planarSizeYScroll->setPosition(123, UP_VECTOR_SLIDER+80);
	planarSizeYScroll->setMinValue(minPlanarY);
	planarSizeYScroll->setMaxValue(maxPlanarY);
	planarSizeYScroll->setValue(10.0f);

	string radius_str			= "\n\n^x0016Radius: %.2f\n";
	MVHSCROLLBAR *radiusScroll = new MVHSCROLLBAR();
	radiusScroll->setSizeX(203);
	radiusScroll->setPosition(22, UP_VECTOR_SLIDER+64);
	radiusScroll->setMinValue(minRadius);
	radiusScroll->setMaxValue(maxRadius);
	radiusScroll->setValue(10.0f);



	// System Parameters Panel 
	string NumberOfParticules_text= "\n^x0016Number of particles: %d\n";
	MVHSCROLLBAR *NbOfPartScroll = new MVHSCROLLBAR();
	NbOfPartScroll->link(parametersWin);
	NbOfPartScroll->setSizeX(203);
	NbOfPartScroll->setPosition(22, NB_PART_SLIDER);
	NbOfPartScroll->setMinValue(0.0f);
	NbOfPartScroll->setMaxValue(4094.0); // 900.0f);
	NbOfPartScroll->setValue(256.0f);

	string Spinning_text= "\n^x0016Spinning: %.2f\n";
	MVHSCROLLBAR *SpinningScroll = new MVHSCROLLBAR();
	SpinningScroll->link(parametersWin);
	SpinningScroll->setSizeX(203);
	SpinningScroll->setPosition(22, SPINNING_SLIDER);
	SpinningScroll->setMinValue(0.0f);
	SpinningScroll->setMaxValue(360.0f);
	SpinningScroll->setValue(0.0f);

	string Position_text		= "\n^x0010 System position\n";
	string positionX_text	= "^x0016X: %.2f\n";
	string positionY_text	= "^x0016Y: %.2f\n";
	string positionZ_text	= "^x0016Z: %.2f\n";
	MVHSCROLLBAR *positionXScroll = new MVHSCROLLBAR();
	positionXScroll->link(parametersWin);
	positionXScroll->setSizeX(120);
	positionXScroll->setPosition(103, POSITION_SLIDER);
	positionXScroll->setMinValue(minPosX);
	positionXScroll->setMaxValue(maxPosX);
	positionXScroll->setValue(0.0f);

	MVHSCROLLBAR *positionYScroll = new MVHSCROLLBAR();
	positionYScroll->link(parametersWin);
	positionYScroll->setSizeX(120);
	positionYScroll->setPosition(103, POSITION_SLIDER+16);
	positionYScroll->setMinValue(minPosY);
	positionYScroll->setMaxValue(maxPosY);
	positionYScroll->setValue(0.0f);

	MVHSCROLLBAR *positionZScroll = new MVHSCROLLBAR();
	positionZScroll->link(parametersWin);
	positionZScroll->setSizeX(120);
	positionZScroll->setPosition(103, POSITION_SLIDER+32);
	positionZScroll->setMinValue(minPosZ);
	positionZScroll->setMaxValue(maxPosZ);
	positionZScroll->setValue(0.0f);

	string Gravity_text		= "\n^x0016Gravity intensity\n";
	string gravityX_text		= "^x0016X: %.2f\n";
	string gravityY_text		= "^x0016Y: %.2f\n";
	string gravityZ_text		= "^x0016Z: %.2f\n";
	MVHSCROLLBAR *gravityXScroll = new MVHSCROLLBAR();
	gravityXScroll->link(parametersWin);
	gravityXScroll->setSizeX(120);
	gravityXScroll->setPosition(103, POSITION_SLIDER+80);
	gravityXScroll->setMinValue(-50.0f);
	gravityXScroll->setMaxValue(50.0f);
	gravityXScroll->setValue(0.0f);

	MVHSCROLLBAR *gravityYScroll = new MVHSCROLLBAR();
	gravityYScroll->link(parametersWin);
	gravityYScroll->setSizeX(120);
	gravityYScroll->setPosition(103, POSITION_SLIDER+96);
	gravityYScroll->setMinValue(-50.0f);
	gravityYScroll->setMaxValue(50.0f);
	gravityYScroll->setValue(0.0f);

	MVHSCROLLBAR *gravityZScroll = new MVHSCROLLBAR();
	gravityZScroll->link(parametersWin);
	gravityZScroll->setSizeX(120);
	gravityZScroll->setPosition(103, POSITION_SLIDER+112);
	gravityZScroll->setMinValue(-50.0f);
	gravityZScroll->setMaxValue(50.0f);
	gravityZScroll->setValue(0.0f);

	string Orientation_text		= "\n^x0010 System orientation\n";
	string orientationX_text	= "^x0016X: %.2f\n";
	string orientationY_text	= "^x0016Y: %.2f\n";
	string orientationZ_text	= "^x0016Z: %.2f\n";
	MVHSCROLLBAR *orientationXScroll = new MVHSCROLLBAR();
	orientationXScroll->link(parametersWin);
	orientationXScroll->setSizeX(120);
	orientationXScroll->setPosition(103, POSITION_SLIDER+160);
	orientationXScroll->setMinValue(-180.0f);
	orientationXScroll->setMaxValue(180.0f);
	orientationXScroll->setValue(0.0f);

	MVHSCROLLBAR *orientationYScroll = new MVHSCROLLBAR();
	orientationYScroll->link(parametersWin);
	orientationYScroll->setSizeX(120);
	orientationYScroll->setPosition(103, POSITION_SLIDER+176);
	orientationYScroll->setMinValue(-180.0f);
	orientationYScroll->setMaxValue(180.0f);
	orientationYScroll->setValue(0.0f);

	MVHSCROLLBAR *orientationZScroll = new MVHSCROLLBAR();
	orientationZScroll->link(parametersWin);
	orientationZScroll->setSizeX(120);
	orientationZScroll->setPosition(103, POSITION_SLIDER+192);
	orientationZScroll->setMinValue(-180.0f);
	orientationZScroll->setMaxValue(180.0f);
	orientationZScroll->setValue(0.0f);

	MVBUTTON *openBlendingPanelButton = new MVBUTTON();
	openBlendingPanelButton->setFont(font);
	openBlendingPanelButton->setCaption("^y0005Blending options");
	openBlendingPanelButton->setSizes(100, 30);
	openBlendingPanelButton->setPosition(20, POSITION_SLIDER+216);
	openBlendingPanelButton->link(parametersWin);

	MVBUTTON *selectTextureButton = new MVBUTTON();
	selectTextureButton->setFont(font);
	selectTextureButton->setCaption("^y0005Select texture...");
	selectTextureButton->setSizes(100, 30);
	selectTextureButton->setPosition(130, POSITION_SLIDER+216);
	selectTextureButton->link(parametersWin);

   // Write text on the Parameters Window
   parametersWin->clear();
   parametersWin->write((char*)NumberOfParticules_text.c_str(), NbOfPartScroll->getValue());
   parametersWin->write((char*)Spinning_text.c_str(), SpinningScroll->getValue());
   parametersWin->write((char*)Position_text.c_str());
   parametersWin->write((char*)positionX_text.c_str(), positionXScroll->getValue());
   parametersWin->write((char*)positionY_text.c_str(), positionYScroll->getValue());
   parametersWin->write((char*)positionZ_text.c_str(), positionZScroll->getValue());

   parametersWin->write((char*)Gravity_text.c_str());
   parametersWin->write((char*)gravityX_text.c_str(), gravityXScroll->getValue());
   parametersWin->write((char*)gravityY_text.c_str(), gravityYScroll->getValue());
   parametersWin->write((char*)gravityZ_text.c_str(), gravityZScroll->getValue());

   parametersWin->write((char*)Orientation_text.c_str());
   parametersWin->write((char*)orientationX_text.c_str(), orientationXScroll->getValue());
   parametersWin->write((char*)orientationY_text.c_str(), orientationYScroll->getValue());
   parametersWin->write((char*)orientationZ_text.c_str(), orientationZScroll->getValue());
   parametersWin->format();

	// Blending options panel
	string		srcBlending_txt = "^x0090Source^x0146Destination";

	gl_zeroSrcButton = new MVBUTTON();
	gl_zeroSrcButton->setFont(font);
	gl_zeroSrcButton->setCaption("^y0001GL_ZERO");
	gl_zeroSrcButton->setSizes(185, 20);
	gl_zeroSrcButton->setPosition(20, 20);
	gl_zeroSrcButton->link(blendingWin);
	gl_zeroSrcButton->setEventType(MV_BUTTON_SWITCH);


	gl_zeroDstButton = new MVBUTTON();
	gl_zeroDstButton->setFont(font);
	gl_zeroDstButton->setCaption("^y0001GL_ZERO");
	gl_zeroDstButton->setSizes(185, 20);
	gl_zeroDstButton->setPosition(210, 20);
	gl_zeroDstButton->link(blendingWin);
	gl_zeroDstButton->setEventType(MV_BUTTON_SWITCH);

	gl_oneSrcButton = new MVBUTTON();
	gl_oneSrcButton->setFont(font);
	gl_oneSrcButton->setCaption("^y0001GL_ONE");
	gl_oneSrcButton->setSizes(185, 20);
	gl_oneSrcButton->setPosition(20, 50);
	gl_oneSrcButton->link(blendingWin);
	gl_oneSrcButton->setEventType(MV_BUTTON_SWITCH);

	gl_oneDstButton = new MVBUTTON();
	gl_oneDstButton->setFont(font);
	gl_oneDstButton->setCaption("^y0001GL_ONE");
	gl_oneDstButton->setSizes(185, 20);
	gl_oneDstButton->setPosition(210, 50);
	gl_oneDstButton->link(blendingWin);
	gl_oneDstButton->setEventType(MV_BUTTON_SWITCH);

	gl_colorSrcButton = new MVBUTTON();
	gl_colorSrcButton->setFont(font);
	gl_colorSrcButton->setCaption("^y0001GL_SRC_COLOR");
	gl_colorSrcButton->setSizes(185, 20);
	gl_colorSrcButton->setPosition(20, 80);
	gl_colorSrcButton->link(blendingWin);
	gl_colorSrcButton->setEventType(MV_BUTTON_SWITCH);

	gl_colorDstButton = new MVBUTTON();
	gl_colorDstButton->setFont(font);
	gl_colorDstButton->setCaption("^y0001GL_DST_COLOR");
	gl_colorDstButton->setSizes(185, 20);
	gl_colorDstButton->setPosition(210, 80);
	gl_colorDstButton->link(blendingWin);
	gl_colorDstButton->setEventType(MV_BUTTON_SWITCH);

	gl_src_SrcAlphaButton = new MVBUTTON();
	gl_src_SrcAlphaButton->setFont(font);
	gl_src_SrcAlphaButton->setCaption("^y0001GL_SRC_ALPHA");
	gl_src_SrcAlphaButton->setSizes(185, 20);
	gl_src_SrcAlphaButton->setPosition(20, 110);
	gl_src_SrcAlphaButton->link(blendingWin);
	gl_src_SrcAlphaButton->setEventType(MV_BUTTON_SWITCH);
	gl_src_SrcAlphaButton->setClickedStatus(1);
	blendingSrcClicked = gl_src_SrcAlphaButton;

	gl_dst_SrcAlphaButton = new MVBUTTON();
	gl_dst_SrcAlphaButton->setFont(font);
	gl_dst_SrcAlphaButton->setCaption("^y0001GL_SRC_ALPHA");
	gl_dst_SrcAlphaButton->setSizes(185, 20);
	gl_dst_SrcAlphaButton->setPosition(210, 110);
	gl_dst_SrcAlphaButton->link(blendingWin);
	gl_dst_SrcAlphaButton->setEventType(MV_BUTTON_SWITCH);
	
	gl_src_DstAlphaButton = new MVBUTTON();
	gl_src_DstAlphaButton->setFont(font);
	gl_src_DstAlphaButton->setCaption("^y0001GL_DST_ALPHA");
	gl_src_DstAlphaButton->setSizes(185, 20);
	gl_src_DstAlphaButton->setPosition(20, 140);
	gl_src_DstAlphaButton->link(blendingWin);
	gl_src_DstAlphaButton->setEventType(MV_BUTTON_SWITCH);

	gl_dst_DstAlphaButton = new MVBUTTON();
	gl_dst_DstAlphaButton->setFont(font);
	gl_dst_DstAlphaButton->setCaption("^y0001GL_DST_ALPHA");
	gl_dst_DstAlphaButton->setSizes(185, 20);
	gl_dst_DstAlphaButton->setPosition(210, 140);
	gl_dst_DstAlphaButton->link(blendingWin);
	gl_dst_DstAlphaButton->setEventType(MV_BUTTON_SWITCH);

	gl_oneMinusDstColorButton = new MVBUTTON();
	gl_oneMinusDstColorButton->setFont(font);
	gl_oneMinusDstColorButton->setCaption("^y0001GL_ONE_MINUS_DST_COLOR");
	gl_oneMinusDstColorButton->setSizes(185, 20);
	gl_oneMinusDstColorButton->setPosition(20, 170);
	gl_oneMinusDstColorButton->link(blendingWin);
	gl_oneMinusDstColorButton->setEventType(MV_BUTTON_SWITCH);

	gl_oneMinusSrcColorButton = new MVBUTTON();
	gl_oneMinusSrcColorButton->setFont(font);
	gl_oneMinusSrcColorButton->setCaption("^y0001GL_ONE_MINUS_SRC_COLOR");
	gl_oneMinusSrcColorButton->setSizes(185, 20);
	gl_oneMinusSrcColorButton->setPosition(210, 170);
	gl_oneMinusSrcColorButton->link(blendingWin);
	gl_oneMinusSrcColorButton->setEventType(MV_BUTTON_SWITCH);

	gl_src_oneMinusSrcAlphaButton = new MVBUTTON();
	gl_src_oneMinusSrcAlphaButton->setFont(font);
	gl_src_oneMinusSrcAlphaButton->setCaption("^y0001GL_ONE_MINUS_SRC_ALPHA");
	gl_src_oneMinusSrcAlphaButton->setSizes(185, 20);
	gl_src_oneMinusSrcAlphaButton->setPosition(20, 200);
	gl_src_oneMinusSrcAlphaButton->link(blendingWin);
	gl_src_oneMinusSrcAlphaButton->setEventType(MV_BUTTON_SWITCH);

	gl_dst_oneMinusSrcAlphaButton = new MVBUTTON();
	gl_dst_oneMinusSrcAlphaButton->setFont(font);
	gl_dst_oneMinusSrcAlphaButton->setCaption("^y0001GL_ONE_MINUS_SRC_ALPHA");
	gl_dst_oneMinusSrcAlphaButton->setSizes(185, 20);
	gl_dst_oneMinusSrcAlphaButton->setPosition(210, 200);
	gl_dst_oneMinusSrcAlphaButton->link(blendingWin);
	gl_dst_oneMinusSrcAlphaButton->setEventType(MV_BUTTON_SWITCH);
	gl_dst_oneMinusSrcAlphaButton->setClickedStatus(1);
	blendingDstClicked = gl_dst_oneMinusSrcAlphaButton;

	gl_src_oneMinusDstAlphaButton = new MVBUTTON();
	gl_src_oneMinusDstAlphaButton->setFont(font);
	gl_src_oneMinusDstAlphaButton->setCaption("^y0001GL_ONE_MINUS_DST_ALPHA");
	gl_src_oneMinusDstAlphaButton->setSizes(185, 20);
	gl_src_oneMinusDstAlphaButton->setPosition(20, 230);
	gl_src_oneMinusDstAlphaButton->link(blendingWin);
	gl_src_oneMinusDstAlphaButton->setEventType(MV_BUTTON_SWITCH);

	gl_dst_oneMinusDstAlphaButton = new MVBUTTON();
	gl_dst_oneMinusDstAlphaButton->setFont(font);
	gl_dst_oneMinusDstAlphaButton->setCaption("^y0001GL_ONE_MINUS_DST_ALPHA");
	gl_dst_oneMinusDstAlphaButton->setSizes(185, 20);
	gl_dst_oneMinusDstAlphaButton->setPosition(210, 230);
	gl_dst_oneMinusDstAlphaButton->link(blendingWin);
	gl_dst_oneMinusDstAlphaButton->setEventType(MV_BUTTON_SWITCH);

	gl_SrcAlphaSaturateButton = new MVBUTTON();
	gl_SrcAlphaSaturateButton->setFont(font);
	gl_SrcAlphaSaturateButton->setCaption("^y0001GL_SRC_ALPHA_SATURATE");
	gl_SrcAlphaSaturateButton->setSizes(185, 20);
	gl_SrcAlphaSaturateButton->setPosition(20, 260);
	gl_SrcAlphaSaturateButton->link(blendingWin);
	gl_SrcAlphaSaturateButton->setEventType(MV_BUTTON_SWITCH);

	blendingWin->clear();
	blendingWin->write((char*)srcBlending_txt.c_str());
	blendingWin->format();
	
	// Remove tex path:
	MVTEXTURE::setPath("");
	
	////////////
	// MAINLOOP:

	int mouseX = 0, mouseY = 0, oldMouseX = 0, oldMouseY = 0;
	bool mouseBL = false, mouseBR = false;
	MVID selected = 0;				// Id of the button selected
	bool occurred = false;			// Say if an event as occured
		

	bool done = false;
	
	while (!done)
	{
		// Clear the buffer:
		MVISIO::clear(true, true, true);

		MVISIO::begin2D();
         PASS_BACKGROUND;
		MVISIO::end2D();

		// Render a 3D scene:
		MVISIO::begin3D(camera);
			if(isEmitterInitialized)
				emitter->pass();
		MVISIO::end3D();       
				

		MVISIO::begin2D();
			displayToolboxPanelButton->pass();
         PASS_QUITBUTTON;
			if(isEmitterInitialized)
			{
				displaySysParametersButton->pass();
				displayParticuleParametersButton->pass();
				if((emitter->getEmitterType() == 1) ||			// and the selected Emitters is planar
			       (emitter->getEmitterType() == 2))			// or circular
				{
					extraParametersButton->pass();				// Then display the extraParams Button
				}
			}
			if(isCreateEmitterWinDisplayed)
				toolboxWin->pass();
			if(isParticuleParamsWinDisplayed)
				particuleParamsWin->pass();
			if(isSystemParameterPanelDisplayed)
				parametersWin->pass();
			if(isExtraParamsWinDisplayed)
			{
				extraParamsWin->pass();
			}
			if(isBlendingPanelDisplayed)
				blendingWin->pass();
		MVISIO::end2D();

		// Check events:			
		SDL_Event _event;
		int buttonState = SDL_GetMouseState(&mouseX, &mouseY);
		if (buttonState&SDL_BUTTON(1))
                    mouseBL = true;                   
            else
                    mouseBL = false;
         if (buttonState&SDL_BUTTON(2) || buttonState&SDL_BUTTON(3))
                    mouseBR = true;                   
            else
                    mouseBR = false;

		while (SDL_PollEvent(&_event)) 
		{
			// Quit by click on X:
			if (_event.type == SDL_QUIT) 
				done = true;	

			// Quit by pressing ESC:
			if (_event.type == SDL_KEYDOWN) 
				if (_event.key.keysym.sym == SDLK_ESCAPE) 
					done = true;									       
		}     

		/**********************/
		/* Check MVISIO Event */
		/**********************/

		// Check MVisio events:
		MVID id=0;
		occurred = MVELEMENT::manageGUIEvents(0, mouseX, mouseY, mouseBL, mouseBR, &id);
		
		if (PRESSED_QUITBUTTON(id))
		{
			      done = true;
	   }
		else if(id == displayToolboxPanelButton->getId())
		{
			isCreateEmitterWinDisplayed = !isCreateEmitterWinDisplayed;
			toolboxWin->setForemost(MVISIO::getTimeStamp());
		}
		else if(id == displayParticuleParametersButton->getId())
		{
			isParticuleParamsWinDisplayed = !isParticuleParamsWinDisplayed;
			particuleParamsWin->setForemost(MVISIO::getTimeStamp());
		}
		else if(id == displaySysParametersButton->getId())
		{
			isSystemParameterPanelDisplayed = !isSystemParameterPanelDisplayed;
			parametersWin->setForemost(MVISIO::getTimeStamp());
		}
		else if(id == openBlendingPanelButton->getId())
		{
			isBlendingPanelDisplayed = !isBlendingPanelDisplayed;
			blendingWin->setForemost(MVISIO::getTimeStamp());
		}
		else if(id == extraParametersButton->getId())
		{
			isExtraParamsWinDisplayed = !isExtraParamsWinDisplayed;
			//planarSizeXScroll->setValue(emitter->getSizeX());
			//planarSizeYScroll->setValue(emitter->getSizeY());
			radiusScroll->setValue(emitter->getRadius());
			extraParamsWin->setForemost(MVISIO::getTimeStamp());
		}
		
		
		// Managage the creation of emitter button
		else if(id == createNewSphericalButton->getId())
		{
			

			int oldEmitterType = -1;
			if(isEmitterInitialized)
			{
				oldEmitterType = emitter->getEmitterType();
				// Save the old values and fill the new emitter with them
		
				// Change the type of the emitter
				emitter->setEmitterType(MV_EMITTER_SPHERICAL);

				if(oldEmitterType == 1)
				{
						planarSizeXScroll->unlink();
						planarSizeYScroll->unlink();
				}
				if(oldEmitterType == 2)
				{
					radiusScroll->unlink();
				}
			}
			else
			{
				emitter = new MVEMITTER();
				emitter->setEmitterType(MV_EMITTER_SPHERICAL);
				emitter->setNumberOfParticles(256);
				emitter->setTexture(tex);
				isEmitterInitialized = true;
										
				// Set the sliders for the System Parameters
				MVVECTOR3 temp = emitter->getPosition();
				positionXScroll->setValue((float)temp.x);
				positionYScroll->setValue((float)temp.y);
				positionZScroll->setValue((float)temp.z);

				temp = emitter->getGravity();
				gravityXScroll->setValue((float)temp.x);
				gravityYScroll->setValue((float)temp.y);
				gravityZScroll->setValue((float)temp.z);

				temp = emitter->getRotation();
				orientationXScroll->setValue(temp.x);
				orientationYScroll->setValue(temp.y);
				orientationZScroll->setValue(temp.z);
				
				NbOfPartScroll->setValue((float)emitter->getNumberOfParticles());
				SpinningScroll->setValue((float)emitter->getSpinning());

				// Set the sliders for the particle Parameters
				energyScroll->setValue((float)emitter->getMaximumEnergy());
				partSizeScroll->setValue((float)emitter->getMaximumParticuleSize());
				velocityScroll->setValue((float)emitter->getMaximumVelocity());
				partSpinScroll->setValue((float)emitter->getMaximumSpin());

				temp = emitter->getColorStart();
				colorMinRedScroll->setValue(temp.x);
				colorMinGreenScroll->setValue(temp.y);
				colorMinBlueScroll->setValue(temp.z);

				temp = emitter->getColorEnd();
				colorMaxRedScroll->setValue(temp.x);
				colorMaxGreenScroll->setValue(temp.y);
				colorMaxBlueScroll->setValue(temp.z);

				setBlendingOptions(emitter->getBlendingSrc(), emitter->getBlendingDst());
			}
			
			// Update of the toolbox windows
			toolboxWin->setSizes(150, 210);
			if(!saveButton->getDisplayStatus())
			{					
				saveButton->setDisplayStatus(true);			
			}
			
			isExtraParamsWinDisplayed = false;

		}
		else if(id == createNewPlanarButton->getId())
		{
			// Insert a new Planar Emitter with default value
			int oldEmitterType = 0;
			if(isEmitterInitialized)
			{
				oldEmitterType = emitter->getEmitterType();
				// Save the old values and fill the new emitter with them
				// Erase the old emitter
				//emitter->setNumberOfParticles(0);		// Required to tell the others emitter that there are new particles free
				//delete emitter;
				if(oldEmitterType == 2)
				{
					radiusScroll->unlink();	
				}
				if(oldEmitterType != 1)
				{
					planarSizeXScroll->link(extraParamsWin);
					planarSizeYScroll->link(extraParamsWin);
				}
				extraParamsWin->clear();
				extraParamsWin->write((char*)upVecStr.c_str());
				extraParamsWin->write((char*)upVec_x.c_str());
				extraParamsWin->write((char*)upVec_y.c_str());
				extraParamsWin->write((char*)upVec_z.c_str());
				extraParamsWin->write((char*)size_x.c_str());
				extraParamsWin->write((char*)size_y.c_str());
				extraParamsWin->format();
				
				emitter->setEmitterType(MV_EMITTER_PLANAR);
				
			}
			else
			{
				emitter = new MVEMITTER();
				emitter->setEmitterType(MV_EMITTER_PLANAR);
				emitter->setNumberOfParticles(256);
				emitter->setUpVector(MVVECTOR3(0.0f, -1.0f, 0.0f));
				emitter->setSizeX(100.0f);
				emitter->setSizeY(100.0f);
				isEmitterInitialized = true;
				emitter->setNumberOfParticles(0);

				emitter->setTexture(tex);

				// Set the sliders for the System Parameters
				MVVECTOR3 temp = emitter->getCenter();
				positionXScroll->setValue((float)temp.x);
				positionYScroll->setValue((float)temp.y);
				positionZScroll->setValue((float)temp.z);

				temp = emitter->getGravity();
				gravityXScroll->setValue((float)temp.x);
				gravityYScroll->setValue((float)temp.y);
				gravityZScroll->setValue((float)temp.z);

				temp = emitter->getRotation();
				orientationXScroll->setValue(temp.x);
				orientationYScroll->setValue(temp.y);
				orientationZScroll->setValue(temp.z);
				
				NbOfPartScroll->setValue((float)emitter->getNumberOfParticles());
				SpinningScroll->setValue((float)emitter->getSpinning());

				// Set the sliders for the particle Parameters
				energyScroll->setValue((float)emitter->getMaximumEnergy());
				partSizeScroll->setValue((float)emitter->getMaximumParticuleSize());
				velocityScroll->setValue((float)emitter->getMaximumVelocity());
				partSpinScroll->setValue((float)emitter->getMaximumSpin());

				temp = emitter->getColorStart();
				colorMinRedScroll->setValue(temp.x);
				colorMinGreenScroll->setValue(temp.y);
				colorMinBlueScroll->setValue(temp.z);

				temp = emitter->getColorEnd();
				colorMaxRedScroll->setValue(temp.x);
				colorMaxGreenScroll->setValue(temp.y);
				colorMaxBlueScroll->setValue(temp.z);		

				temp = emitter->getUpVector();
				upVectorScrollX->setValue(temp.x);
				upVectorScrollY->setValue(temp.y);
				upVectorScrollZ->setValue(temp.z);

				planarSizeXScroll->setValue(emitter->getSizeX());
				planarSizeYScroll->setValue(emitter->getSizeY());

				extraParamsWin->clear();
				extraParamsWin->write((char*)upVecStr.c_str());
				extraParamsWin->write((char*)upVec_x.c_str());
				extraParamsWin->write((char*)upVec_y.c_str());
				extraParamsWin->write((char*)upVec_z.c_str());
				extraParamsWin->write((char*)size_x.c_str());
				extraParamsWin->write((char*)size_y.c_str());
				extraParamsWin->format();

				planarSizeXScroll->link(extraParamsWin);
				planarSizeYScroll->link(extraParamsWin);
				
				setBlendingOptions(emitter->getBlendingSrc(), emitter->getBlendingDst());
			}
			
			// Update of the toolbox windows
			if(!saveButton->getDisplayStatus())
			{					
				saveButton->setDisplayStatus(true);			
			}
		}
		else if(id == createNewCircularButton->getId())
		{
			// Insert a new Planar Emitter with default value
			int oldEmitterType = 0;
			if(isEmitterInitialized)
			{
				oldEmitterType = emitter->getEmitterType();
				if(oldEmitterType == 1)
				{
					planarSizeXScroll->unlink();
					planarSizeYScroll->unlink();
				}
				if(oldEmitterType != 2)
				{
					radiusScroll->link(extraParamsWin);
				}

				extraParamsWin->clear();
				extraParamsWin->write((char*)upVecStr.c_str());
				extraParamsWin->write((char*)upVec_x.c_str());
				extraParamsWin->write((char*)upVec_y.c_str());
				extraParamsWin->write((char*)upVec_z.c_str());
				extraParamsWin->write((char*)radius_str.c_str());
				extraParamsWin->format();

				emitter->setEmitterType(MV_EMITTER_CIRCULAR);
			}
			else
			{
				emitter = new MVEMITTER();
				emitter->setEmitterType(MV_EMITTER_CIRCULAR);
				emitter->setNumberOfParticles(256);
				emitter->setRadius(10.0f);
				isEmitterInitialized = true;
				emitter->setTexture(tex);

				// Set the sliders for the System Parameters
				MVVECTOR3 temp = emitter->getCenter();
				positionXScroll->setValue((float)temp.x);
				positionYScroll->setValue((float)temp.y);
				positionZScroll->setValue((float)temp.z);

				temp = emitter->getGravity();
				gravityXScroll->setValue((float)temp.x);
				gravityYScroll->setValue((float)temp.y);
				gravityZScroll->setValue((float)temp.z);

				temp = emitter->getRotation();
				orientationXScroll->setValue(temp.x);
				orientationYScroll->setValue(temp.y);
				orientationZScroll->setValue(temp.z);
				
				NbOfPartScroll->setValue((float)emitter->getNumberOfParticles());
				SpinningScroll->setValue((float)emitter->getSpinning());

				// Set the sliders for the particle Parameters
				energyScroll->setValue((float)emitter->getMaximumEnergy());
				partSizeScroll->setValue((float)emitter->getMaximumParticuleSize());
				velocityScroll->setValue((float)emitter->getMaximumVelocity());
				partSpinScroll->setValue((float)emitter->getMaximumSpin());

				temp = emitter->getColorStart();
				colorMinRedScroll->setValue(temp.x);
				colorMinGreenScroll->setValue(temp.y);
				colorMinBlueScroll->setValue(temp.z);

				temp = emitter->getColorEnd();
				colorMaxRedScroll->setValue(temp.x);
				colorMaxGreenScroll->setValue(temp.y);
				colorMaxBlueScroll->setValue(temp.z);		

				temp = emitter->getUpVector();
				upVectorScrollX->setValue(temp.x);
				upVectorScrollY->setValue(temp.y);
				upVectorScrollZ->setValue(temp.z);

				radiusScroll->setValue(emitter->getRadius());
							
				extraParamsWin->clear();
				extraParamsWin->write((char*)upVecStr.c_str());
				extraParamsWin->write((char*)upVec_x.c_str());
				extraParamsWin->write((char*)upVec_y.c_str());
				extraParamsWin->write((char*)upVec_z.c_str());
				extraParamsWin->write((char*)radius_str.c_str());
				extraParamsWin->format();			

				radiusScroll->link(extraParamsWin);

				setBlendingOptions(emitter->getBlendingSrc(), emitter->getBlendingDst());
			}

			// Update of the toolbox windows
			if(!saveButton->getDisplayStatus())
			{	
				saveButton->setDisplayStatus(true);				
			}
		}
		else if(id == loadButton->getId())
		{
			ofn.Flags = OFN_EXPLORER | OFN_PATHMUSTEXIST |OFN_FILEMUSTEXIST;
			ofn.lpstrTitle = TEXT("Load an Emitter Scene from a XML file.");
			ofn.lpstrDefExt = TEXT("xml");
			ofn.lpstrFilter = TEXT("XML Files (*.xml)\0*.xml\0");
			if (GetOpenFileName(&ofn)) {
				

				// Insert a new Planar Emitter with default value
				int oldEmitterType = 0;
				if(isEmitterInitialized)
				{
					oldEmitterType = emitter->getEmitterType();
					// Save the old values and fill the new emitter with them
					// Erase the old emitter
					emitter->setNumberOfParticles(0);		// Required to tell the others emitter that there are new particles free
					delete emitter;
				}
				emitter = new MVEMITTER();
				emitter->load(fileName);
				isEmitterInitialized = true;
				if(emitter->getEmitterType() == 0)
				{
					isExtraParamsWinDisplayed = false;
				}
				else if(emitter->getEmitterType() == 1)
				{
					extraParamsWin->clear();
					extraParamsWin->write((char*)upVecStr.c_str());
					extraParamsWin->write((char*)upVec_x.c_str(), upVectorScrollX->getValue());
					extraParamsWin->write((char*)upVec_y.c_str(), upVectorScrollY->getValue());
					extraParamsWin->write((char*)upVec_z.c_str(), upVectorScrollZ->getValue());
					extraParamsWin->write((char*)size_x.c_str(), planarSizeXScroll->getValue());
					extraParamsWin->write((char*)size_y.c_str(), planarSizeYScroll->getValue());
					extraParamsWin->format();
					if(oldEmitterType !=1)
					{
						planarSizeXScroll->link(extraParamsWin);
						planarSizeYScroll->link(extraParamsWin);
					}
					if(oldEmitterType == 2)
					{
						radiusScroll->unlink();
					}
				}
				else if(emitter->getEmitterType() == 2)
				{
					extraParamsWin->clear();
					extraParamsWin->write((char*)upVecStr.c_str());
					extraParamsWin->write((char*)upVec_x.c_str(), upVectorScrollX->getValue());
					extraParamsWin->write((char*)upVec_y.c_str(), upVectorScrollY->getValue());
					extraParamsWin->write((char*)upVec_z.c_str(), upVectorScrollZ->getValue());
					extraParamsWin->write((char*)radius_str.c_str(), radiusScroll->getValue());
					extraParamsWin->format();
					if(oldEmitterType !=2)
					{
						radiusScroll->link(extraParamsWin);
					}
					if(oldEmitterType == 1)
					{
						planarSizeXScroll->unlink();
						planarSizeYScroll->unlink();
					}
				}
				
				// Set the sliders for the System Parameters
				MVVECTOR3 temp = emitter->getCenter();
				positionXScroll->setValue((float)temp.x);
				positionYScroll->setValue((float)temp.y);
				positionZScroll->setValue((float)temp.z);

				temp = emitter->getGravity();
				gravityXScroll->setValue((float)temp.x);
				gravityYScroll->setValue((float)temp.y);
				gravityZScroll->setValue((float)temp.z);

				temp = emitter->getRotation();
				orientationXScroll->setValue(temp.x);
				orientationYScroll->setValue(temp.y);
				orientationZScroll->setValue(temp.z);
				
				NbOfPartScroll->setValue((float)emitter->getNumberOfParticles());
				SpinningScroll->setValue((float)emitter->getSpinning());

				// Set the sliders for the particle Parameters
				energyScroll->setValue((float)emitter->getMaximumEnergy());
				partSizeScroll->setValue((float)emitter->getMaximumParticuleSize());
				velocityScroll->setValue((float)emitter->getMaximumVelocity());
				partSpinScroll->setValue((float)emitter->getMaximumSpin());

				temp = emitter->getColorStart();
				colorMinRedScroll->setValue(temp.x);
				colorMinGreenScroll->setValue(temp.y);
				colorMinBlueScroll->setValue(temp.z);

				temp = emitter->getColorEnd();
				colorMaxRedScroll->setValue(temp.x);
				colorMaxGreenScroll->setValue(temp.y);
				colorMaxBlueScroll->setValue(temp.z);		

				temp = emitter->getUpVector();
				upVectorScrollX->setValue(temp.x);
				upVectorScrollY->setValue(temp.y);
				upVectorScrollZ->setValue(temp.z);

				planarSizeXScroll->setValue(emitter->getSizeX());
				planarSizeYScroll->setValue(emitter->getSizeY());

				setBlendingOptions(emitter->getBlendingSrc(), emitter->getBlendingDst());
				
				if(!saveButton->getDisplayStatus())
				{				
					saveButton->setDisplayStatus(true);				
				}
			}
		}
		else if(id == saveButton->getId())
		{
			ofn.Flags = OFN_EXPLORER | OFN_PATHMUSTEXIST;
			ofn.lpstrInitialDir = TEXT("C:\\");
			ofn.lpstrDefExt = TEXT("xml");
			ofn.lpstrTitle = TEXT("Save the Scene to a XML file.");
			ofn.lpstrFilter = TEXT("XML Files (*.xml)\0*.xml\0");
			if (GetSaveFileName(&ofn)) {
				saveEmitterAsXML(fileName);
			}
		}
		else if(id == selectTextureButton->getId())
		{
			ofn.Flags = OFN_EXPLORER | OFN_PATHMUSTEXIST |OFN_FILEMUSTEXIST;
			ofn.lpstrTitle = TEXT("Select a texture");
			ofn.lpstrDefExt = TEXT(".tga");
			ofn.lpstrFilter = TEXT("Targea Files (*.tga)\0*.tga\0");
			if (GetOpenFileName(&ofn)) 
			{			 
			   MVTEXTURE *_temp = emitter->getTexture();
				MVTEXTURE *temp = new MVTEXTURE();						// Temporary pointer to a texture
				temp->load(fileName);									// Load the new texture
				emitter->setTexture(temp);								// Apply it to the selectedEmitter				
				delete _temp;
			}
		}
		else if(id == gl_zeroSrcButton->getId())
		{
			emitter->setBlendingSrc(GL_ZERO);
			if(blendingSrcClicked->getId() != gl_zeroSrcButton->getId())
				blendingSrcClicked->setClickedStatus(0);			// Erase the old cliked button status
			else 
				blendingSrcClicked->setClickedStatus(1);			// Reactivate the button to cancel its comportement
			blendingSrcClicked = gl_zeroSrcButton;
		}
		else if(id == gl_zeroDstButton->getId())
		{
			emitter->setBlendingDst(GL_ZERO);
			if(blendingDstClicked->getId() != gl_zeroDstButton->getId())
				blendingDstClicked->setClickedStatus(0);			// Erase the old cliked button status
			else 
				blendingDstClicked->setClickedStatus(1);			// Reactivate the button to cancel its comportement
			blendingDstClicked = gl_zeroDstButton;
		}
		else if(id == gl_oneSrcButton->getId())
		{
			emitter->setBlendingSrc(GL_ONE);
			if(blendingSrcClicked->getId() != gl_oneSrcButton->getId())
				blendingSrcClicked->setClickedStatus(0);			// Erase the old cliked button status
			else 
				blendingSrcClicked->setClickedStatus(1);			// Reactivate the button to cancel its comportement
			blendingSrcClicked = gl_oneSrcButton;
		}
		else if(id == gl_oneDstButton->getId())
		{
			emitter->setBlendingDst(GL_ONE);
			if(blendingDstClicked->getId() != gl_oneDstButton->getId())
				blendingDstClicked->setClickedStatus(0);			// Erase the old cliked button status
			else 
				blendingDstClicked->setClickedStatus(1);			// Reactivate the button to cancel its comportement
			blendingDstClicked = gl_oneDstButton;
		}
		else if(id == gl_colorSrcButton->getId())
		{
			emitter->setBlendingSrc(GL_SRC_COLOR);
			if(blendingSrcClicked->getId() != gl_colorSrcButton->getId())
				blendingSrcClicked->setClickedStatus(0);			// Erase the old cliked button status
			else 
				blendingSrcClicked->setClickedStatus(1);			// Reactivate the button to cancel its comportement
			blendingSrcClicked = gl_colorSrcButton;
		}
		else if(id == gl_colorDstButton->getId())
		{
			emitter->setBlendingDst(GL_DST_COLOR);
			if(blendingDstClicked->getId() != gl_colorDstButton->getId())
				blendingDstClicked->setClickedStatus(0);			// Erase the old cliked button status
			else 
				blendingDstClicked->setClickedStatus(1);			// Reactivate the button to cancel its comportement
			blendingDstClicked = gl_colorDstButton;
		}
		else if(id == gl_src_SrcAlphaButton->getId())
		{
			emitter->setBlendingSrc(GL_SRC_ALPHA);
			if(blendingSrcClicked->getId() != gl_src_SrcAlphaButton->getId())
				blendingSrcClicked->setClickedStatus(0);			// Erase the old cliked button status
			else 
				blendingSrcClicked->setClickedStatus(1);			// Reactivate the button to cancel its comportement
			blendingSrcClicked = gl_src_SrcAlphaButton;
		}
		else if(id == gl_dst_SrcAlphaButton->getId())
		{
			emitter->setBlendingDst(GL_SRC_ALPHA);
			if(blendingDstClicked->getId() != gl_dst_SrcAlphaButton->getId())
				blendingDstClicked->setClickedStatus(0);			// Erase the old cliked button status
			else 
				blendingDstClicked->setClickedStatus(1);			// Reactivate the button to cancel its comportement
			blendingDstClicked = gl_dst_SrcAlphaButton;
		}
		else if(id == gl_src_DstAlphaButton->getId())
		{
			emitter->setBlendingSrc(GL_DST_ALPHA);
			if(blendingSrcClicked->getId() != gl_src_DstAlphaButton->getId())
				blendingSrcClicked->setClickedStatus(0);			// Erase the old cliked button status
			else 
				blendingSrcClicked->setClickedStatus(1);			// Reactivate the button to cancel its comportement
			blendingSrcClicked = gl_src_DstAlphaButton;
		}
		else if(id == gl_dst_DstAlphaButton->getId())
		{
			emitter->setBlendingDst(GL_DST_ALPHA);
			if(blendingDstClicked->getId() != gl_dst_DstAlphaButton->getId())
				blendingDstClicked->setClickedStatus(0);			// Erase the old cliked button status
			else 
				blendingDstClicked->setClickedStatus(1);			// Reactivate the button to cancel its comportement
			blendingDstClicked = gl_dst_DstAlphaButton;
		}
		else if(id == gl_oneMinusDstColorButton->getId())
		{
			emitter->setBlendingSrc(GL_ONE_MINUS_DST_COLOR);
			if(blendingSrcClicked->getId() != gl_oneMinusDstColorButton->getId())
				blendingSrcClicked->setClickedStatus(0);			// Erase the old cliked button status
			else 
				blendingSrcClicked->setClickedStatus(1);			// Reactivate the button to cancel its comportement
			blendingSrcClicked = gl_oneMinusDstColorButton;
		}
		else if(id == gl_oneMinusSrcColorButton->getId())
		{
			emitter->setBlendingDst(GL_ONE_MINUS_SRC_COLOR);
			if(blendingDstClicked->getId() != gl_oneMinusSrcColorButton->getId())
				blendingDstClicked->setClickedStatus(0);			// Erase the old cliked button status
			else 
				blendingDstClicked->setClickedStatus(1);			// Reactivate the button to cancel its comportement
			blendingDstClicked = gl_oneMinusSrcColorButton;
		}
		else if(id == gl_src_oneMinusSrcAlphaButton->getId())
		{
			emitter->setBlendingSrc(GL_ONE_MINUS_SRC_ALPHA);
			if(blendingSrcClicked->getId() != gl_src_oneMinusSrcAlphaButton->getId())
				blendingSrcClicked->setClickedStatus(0);			// Erase the old cliked button status
			else 
				blendingSrcClicked->setClickedStatus(1);			// Reactivate the button to cancel its comportement
			blendingSrcClicked = gl_src_oneMinusSrcAlphaButton;
		}
		else if(id == gl_dst_oneMinusSrcAlphaButton->getId())
		{
			emitter->setBlendingDst(GL_ONE_MINUS_SRC_ALPHA);
			if(blendingDstClicked->getId() != gl_dst_oneMinusSrcAlphaButton->getId())
				blendingDstClicked->setClickedStatus(0);			// Erase the old cliked button status
			else 
				blendingDstClicked->setClickedStatus(1);			// Reactivate the button to cancel its comportement
			blendingDstClicked = gl_dst_oneMinusSrcAlphaButton;
		}
		else if(id == gl_src_oneMinusDstAlphaButton->getId())
		{
			emitter->setBlendingSrc(GL_ONE_MINUS_DST_ALPHA);
			if(blendingSrcClicked->getId() != gl_src_oneMinusDstAlphaButton->getId())
				blendingSrcClicked->setClickedStatus(0);			// Erase the old cliked button status
			else 
				blendingSrcClicked->setClickedStatus(1);			// Reactivate the button to cancel its comportement
			blendingSrcClicked = gl_src_oneMinusDstAlphaButton;
		}
		else if(id == gl_dst_oneMinusDstAlphaButton->getId())
		{
			emitter->setBlendingDst(GL_ONE_MINUS_DST_ALPHA);
			if(blendingDstClicked->getId() != gl_dst_oneMinusDstAlphaButton->getId())
				blendingDstClicked->setClickedStatus(0);			// Erase the old cliked button status
			else 
				blendingDstClicked->setClickedStatus(1);			// Reactivate the button to cancel its comportement
			blendingDstClicked = gl_dst_oneMinusDstAlphaButton;
		}
		else if(id == gl_SrcAlphaSaturateButton->getId())
		{
			emitter->setBlendingSrc(GL_SRC_ALPHA_SATURATE);
			if(blendingSrcClicked->getId() != gl_SrcAlphaSaturateButton->getId())
				blendingSrcClicked->setClickedStatus(0);			// Erase the old cliked button status
			else 
				blendingSrcClicked->setClickedStatus(1);			// Reactivate the button to cancel its comportement
			blendingSrcClicked = gl_SrcAlphaSaturateButton;
		}
		// Manage the windows comportements
		
		if(isParticuleParamsWinDisplayed)
		{	
			// Update the maximum energy of the particules
			emitter->setMaximumEnergy((unsigned)energyScroll->getValue());

			// Update the maximum size of the particules
			emitter->setMaximumParticuleSize((unsigned)partSizeScroll->getValue());

			// Update the maximum weight of the particles
			emitter->setMaximumVelocity((unsigned)velocityScroll->getValue());

			//Update the spin of the particles
			emitter->setMaximumSpin((unsigned)partSpinScroll->getValue());

			// Update the color Range Min value
			emitter->setColorStart(MVVECTOR3(colorMinRedScroll->getValue(), 
											 colorMinGreenScroll->getValue(), 
											 colorMinBlueScroll->getValue()));
			
			// Update the color Range Max value
			emitter->setColorEnd(MVVECTOR3( colorMaxRedScroll->getValue(), 
											colorMaxGreenScroll->getValue(),
											colorMaxBlueScroll->getValue()));
											
			// Write text on the colorRange Window
	      particuleParamsWin->clear();
	      particuleParamsWin->write((char*)Energy_text.c_str(), (int)energyScroll->getValue());
	      particuleParamsWin->write((char*)Size_text.c_str(), (int)partSizeScroll->getValue());
	      particuleParamsWin->write((char*)Velocity_text.c_str(), (int)velocityScroll->getValue());
	      particuleParamsWin->write((char*)Spin_text.c_str(), (int)partSpinScroll->getValue());
	      particuleParamsWin->write((char*)colorStart.c_str());
	      particuleParamsWin->write((char*)colorRange_r.c_str(), colorMinRedScroll->getValue());
	      particuleParamsWin->write((char*)colorRange_g.c_str(), colorMinGreenScroll->getValue());
	      particuleParamsWin->write((char*)colorRange_b.c_str(), colorMinBlueScroll->getValue());
	      particuleParamsWin->write((char*)colorEnd.c_str());
	      particuleParamsWin->write((char*)colorRange_r.c_str(), colorMaxRedScroll->getValue());
	      particuleParamsWin->write((char*)colorRange_g.c_str(), colorMaxGreenScroll->getValue());
	      particuleParamsWin->write((char*)colorRange_b.c_str(), colorMaxBlueScroll->getValue());
	      particuleParamsWin->format();
		}
		if(isSystemParameterPanelDisplayed)
		{
			 // Update the maximum number of particle alive
			//NbOfPartScroll->setValue(0.0f);
			emitter->setNumberOfParticles((int)(NbOfPartScroll->getValue()));

			 // Update the spinning of the particule
			 emitter->setSpinning(SpinningScroll->getValue());

			 // Update the center of the particule System
			 emitter->setPosition(	MVVECTOR3(positionXScroll->getValue(), 
									positionYScroll->getValue(), 
									positionZScroll->getValue()));
			
			 // Update the gravity of the system
			 emitter->setGravity(MVVECTOR3(	gravityXScroll->getValue(), 
											gravityYScroll->getValue(), 
											gravityZScroll->getValue()));

			 emitter->setRotation(	orientationXScroll->getValue(),
									orientationYScroll->getValue(),
									orientationZScroll->getValue());
									
			// Write text on the Parameters Window
	      parametersWin->clear();	     
	      parametersWin->write((char*)NumberOfParticules_text.c_str(), (int)NbOfPartScroll->getValue());
	      parametersWin->write((char*)Spinning_text.c_str(), SpinningScroll->getValue());
	      parametersWin->write((char*)Position_text.c_str());
	      parametersWin->write((char*)positionX_text.c_str(), positionXScroll->getValue());
	      parametersWin->write((char*)positionY_text.c_str(), positionYScroll->getValue());
	      parametersWin->write((char*)positionZ_text.c_str(), positionZScroll->getValue());

	      parametersWin->write((char*)Gravity_text.c_str());
	      parametersWin->write((char*)gravityX_text.c_str(), gravityXScroll->getValue());
	      parametersWin->write((char*)gravityY_text.c_str(), gravityYScroll->getValue());
	      parametersWin->write((char*)gravityZ_text.c_str(), gravityZScroll->getValue());

	      parametersWin->write((char*)Orientation_text.c_str());
	      parametersWin->write((char*)orientationX_text.c_str(), orientationXScroll->getValue());
	      parametersWin->write((char*)orientationY_text.c_str(), orientationYScroll->getValue());
	      parametersWin->write((char*)orientationZ_text.c_str(), orientationZScroll->getValue());	      
	      parametersWin->format();
		}
		if(isExtraParamsWinDisplayed)
		{	
			//emitter->setRotation(0.0f, 0.0f, maxAngleScroll->getValue());

			emitter->setUpVector(MVVECTOR3(	upVectorScrollX->getValue(), 
											upVectorScrollY->getValue(),
											upVectorScrollZ->getValue()));
			
			emitter->setSizeX(planarSizeXScroll->getValue());
			emitter->setSizeY(planarSizeYScroll->getValue());

			// Update the radius
			emitter->setRadius(radiusScroll->getValue());
			
			if(emitter->getEmitterType() == 1)
			{
			      extraParamsWin->clear();
					extraParamsWin->write((char*)upVecStr.c_str());
					extraParamsWin->write((char*)upVec_x.c_str(), upVectorScrollX->getValue());
					extraParamsWin->write((char*)upVec_y.c_str(), upVectorScrollY->getValue());
					extraParamsWin->write((char*)upVec_z.c_str(), upVectorScrollZ->getValue());
					extraParamsWin->write((char*)size_x.c_str(), planarSizeXScroll->getValue());
					extraParamsWin->write((char*)size_y.c_str(), planarSizeYScroll->getValue());
					extraParamsWin->format();
			}
		   else if(emitter->getEmitterType() == 2)
		   {
		         extraParamsWin->clear();
					extraParamsWin->write((char*)upVecStr.c_str());
					extraParamsWin->write((char*)upVec_x.c_str(), upVectorScrollX->getValue());
					extraParamsWin->write((char*)upVec_y.c_str(), upVectorScrollY->getValue());
					extraParamsWin->write((char*)upVec_z.c_str(), upVectorScrollZ->getValue());
					extraParamsWin->write((char*)radius_str.c_str(), radiusScroll->getValue());
					extraParamsWin->format();
		   }
			
		}
		// Swap back to front buffer:    
		MVISIO::swap(); 
	}	

	// Release MVisio:
	delete tex;
	clean();
	MVISIO::free();
	
	// Done:
	return 0;
}

void setBlendingOptions(int src, int dst)
{
	switch(src)
		{
		case 0:
			blendingSrcClicked->setClickedStatus(false);
			gl_zeroSrcButton->setClickedStatus(true);
			blendingSrcClicked = gl_zeroSrcButton;
			break;
		case 1:
			blendingSrcClicked->setClickedStatus(false);
			gl_oneSrcButton->setClickedStatus(true);
			blendingSrcClicked = gl_oneSrcButton;
			break;
		case 768:
			blendingSrcClicked->setClickedStatus(false);
			gl_colorSrcButton->setClickedStatus(true);
			blendingSrcClicked = gl_colorSrcButton;
			break;
		case 770:
			blendingSrcClicked->setClickedStatus(false);
			gl_src_SrcAlphaButton->setClickedStatus(true);
			blendingSrcClicked = gl_src_SrcAlphaButton;
			break;
		case 772:
			blendingSrcClicked->setClickedStatus(false);
			gl_src_DstAlphaButton->setClickedStatus(true);
			blendingSrcClicked = gl_src_DstAlphaButton;
			break;
		case 775:
			blendingSrcClicked->setClickedStatus(false);
			gl_oneMinusDstColorButton->setClickedStatus(true);
			blendingSrcClicked = gl_oneMinusDstColorButton;
			break;
		case 771:
			blendingSrcClicked->setClickedStatus(false);
			gl_src_oneMinusSrcAlphaButton->setClickedStatus(true);
			blendingSrcClicked = gl_src_oneMinusSrcAlphaButton;
			break;
		case 773:
			blendingSrcClicked->setClickedStatus(false);
			gl_src_oneMinusDstAlphaButton->setClickedStatus(true);
			blendingSrcClicked = gl_src_oneMinusDstAlphaButton;
			break;
		case 776:
			blendingSrcClicked->setClickedStatus(false);
			gl_SrcAlphaSaturateButton->setClickedStatus(true);
			blendingSrcClicked = gl_SrcAlphaSaturateButton;
			break;
		default: 
			blendingSrcClicked->setClickedStatus(false);
			gl_oneSrcButton->setClickedStatus(true);
			blendingSrcClicked = gl_oneSrcButton;
			break;
		}
		switch(dst)
		{
		case 0:
			blendingDstClicked->setClickedStatus(false);
			gl_zeroDstButton->setClickedStatus(true);
			blendingDstClicked = gl_zeroDstButton;
			break;
		case 1:
			blendingDstClicked->setClickedStatus(false);
			gl_oneDstButton->setClickedStatus(true);
			blendingDstClicked = gl_oneDstButton;
			break;
		case 774:
			blendingDstClicked->setClickedStatus(false);
			gl_colorDstButton->setClickedStatus(true);
			blendingDstClicked = gl_colorDstButton;
			break;
		case 770:
			blendingDstClicked->setClickedStatus(false);
			gl_dst_SrcAlphaButton->setClickedStatus(true);
			blendingDstClicked = gl_dst_SrcAlphaButton;
			break;
		case 772:
			blendingDstClicked->setClickedStatus(false);
			gl_dst_DstAlphaButton->setClickedStatus(true);
			blendingDstClicked = gl_dst_DstAlphaButton;
			break;
		case 769:
			blendingDstClicked->setClickedStatus(false);
			gl_oneMinusSrcColorButton->setClickedStatus(true);
			blendingDstClicked = gl_oneMinusSrcColorButton;
			break;
		case 771:
			blendingDstClicked->setClickedStatus(false);
			gl_dst_oneMinusSrcAlphaButton->setClickedStatus(true);
			blendingDstClicked = gl_dst_oneMinusSrcAlphaButton;
			break;
		case 773:
			blendingDstClicked->setClickedStatus(false);
			gl_dst_oneMinusDstAlphaButton->setClickedStatus(true);
			blendingDstClicked = gl_dst_oneMinusDstAlphaButton;
			break;
		default: 
			blendingDstClicked->setClickedStatus(false);
			gl_colorDstButton->setClickedStatus(true);
			blendingDstClicked = gl_colorDstButton;
			break;
		}
}

bool loadSetup(const char* filename)
{
	char str[15];
	float value;

	FILE* file;
	file = fopen(filename, "r");

	if(file==NULL)
		return false;

	while((fscanf(file, "%s %f\n", str, &value))!=EOF)
	{
		if(strcmp(str, "MIN_POS_X") == 0)
		{
			minPosX = value;
		}
		else if(strcmp(str, "MAX_POS_X") == 0)
		{
			maxPosX = value;
		}
		else if(strcmp(str, "MIN_POS_Y") == 0)
		{
			minPosY = value;
		}
		else if(strcmp(str, "MAX_POS_Y") == 0)
		{
			maxPosY = value;
		}
		else if(strcmp(str, "MIN_POS_Z") == 0)
		{
			minPosZ = value;
		}
		else if(strcmp(str, "MAX_POS_Z") == 0)
		{
			maxPosZ = value;
		}
		else if(strcmp(str, "MAX_ENERGY") == 0)
		{
		   if(value > 0)
		      maxEnergy = value;
		   else
		      maxEnergy = 200.0f;
		}
		else if(strcmp(str, "MIN_RADIUS") == 0)
		{
			if(value >=0)
				minRadius = value;
			else
				minRadius = 1.0f;
		}
		else if(strcmp(str, "MAX_RADIUS") == 0)
		{
			if(value >= 0)
				maxRadius = value;
			else
				maxRadius = 100.0f;
		}
		else if(strcmp(str, "MIN_PLANAR_X") == 0)
		{
			if(value >=0)
				minPlanarX = value;
			else
				minPlanarX = 1.0f;			
		}
		else if(strcmp(str, "MAX_PLANAR_X") == 0)
		{
			if(value >=0)
				maxPlanarX = value;
			else
				maxPlanarX = 100.0f;			
		}
		else if(strcmp(str, "MIN_PLANAR_Y") == 0)
		{
			if(value >=0)
				minPlanarY = value;
			else
				minPlanarY = 1.0f;			
		}
		else if(strcmp(str, "MAX_PLANAR_Y") == 0)
		{
			if(value >=0)
				maxPlanarY = value;
			else
				maxPlanarY = 100.0f;			
		}
		else if(strcmp(str, "MIN_PART_SIZE") == 0)
		{
			if(value >=0)
				minParticleSize = value;
			else
				minParticleSize = 0.0f;
		}
		else if(strcmp(str, "MAX_PART_SIZE") == 0)
		{
			if(value >=0)
				maxParticleSize = value;
			else
				maxParticleSize = 0.0f;
		}
		
	}
	fclose(file);
	return true;
}


bool clean()
{
	delete emitter;
	return true;
}

// Save the selected emitter as an XML file
bool saveEmitterAsXML(char *filename)
{
	FILE* file;
	file = fopen(filename, "w");
	MVVECTOR3 temp;					// A temporary vector for complicated manipulation	
	
	fprintf(file, "<emitter>\n");															// New partciles system
	fprintf(file, "<type value=\"%d\" />\n", emitter->getEmitterType());				// Get the type of emitter

	// Get the the position of the last occurence of "\" into the filename
	char *str = emitter->getTexture()->getFilename();
	char *pch;
	string fname = str;				// just create a string to have access to the substr function

	pch=strrchr(str,'\\');			// Find the index of the last occurence of "\"
	if(pch != NULL)
	{
		__w64 int index = pch-str+1;			// Compute the index according to the pointers
		printf("%s\n", fname.c_str());
		printf("index: %d, size: %d\n", index, fname.size());
		fname = fname.substr(index);	// Get the substring containing the filnename
	}
		
	fprintf(file, "<texture value=\"%s\" />\n", fname.c_str());						// Get the filename of the texture
	fprintf(file, "<numberOfParticles value=\"%d\" />\n", emitter->getNumberOfParticles());// Get the number of particles
	fprintf(file, "<maxEnergy value=\"%d\" />\n", emitter->getMaximumEnergy());	    // Get the maximum of energy
	fprintf(file, "<maxVelocity value=\"%d\" />\n", emitter->getMaximumVelocity());	// Get the maximum velocity
	fprintf(file, "<emitterSize x=\"%f\" y=\"%f\" />\n", emitter->getSizeX(), emitter->getSizeY());
	fprintf(file, "<radius value=\"%f\" />\n", emitter->getRadius());				// Get the radius of a circular emitter
	fprintf(file, "<maxParticleSize value=\"%d\" />\n", emitter->getMaximumParticuleSize());// Get the maximum size
	fprintf(file, "<maxSpin value=\"%d\" />\n", emitter->getMaximumSpin());			// Get the maximum spin
	fprintf(file, "<spinning value=\"%f\" />\n", emitter->getSpinning());			// Get the spinning

	temp = emitter->getPosition();
	fprintf(file, "<center x=\"%f\" y=\"%f\" z=\"%f\" />\n", temp.x, temp.y, temp.z);// Get the gravity

	temp = emitter->getGravity();
	fprintf(file, "<gravity x=\"%f\" y=\"%f\" z=\"%f\" />\n", temp.x, temp.y, temp.z);// Get the gravity

	temp = emitter->getRotation();
	fprintf(file, "<rotation x=\"%f\" y=\"%f\" z=\"%f\" />\n", temp.x, temp.y, temp.z);// Get the orientation of the system

	temp = emitter->getUpVector();
	fprintf(file, "<upVector x=\"%f\" y=\"%f\" z=\"%f\" />\n", temp.x, temp.y, temp.z);

	temp = emitter->getColorStart();
	fprintf(file, "<colorStart x=\"%f\" y=\"%f\" z=\"%f\" />\n", temp.x, temp.y, temp.z);// Get the color range minimum

	temp = emitter->getColorEnd();
	fprintf(file, "<colorEnd x=\"%f\" y=\"%f\" z=\"%f\" />\n", temp.x, temp.y, temp.z);// Get the color range minimum
	fprintf(file, "<blendingSrc value=\"%d\" />\n", emitter->getBlendingSrc());			// Get the Blending Source option
	fprintf(file, "<blendingDst value=\"%d\" />\n", emitter->getBlendingDst());			// Get the Blending Destination option
	fprintf(file, "</emitter>\n");														// End of the new partciles system
	fclose(file);
	return true;
}